@@ -66,6 +66,8 @@
 #define E_CALENDAR_AUTO_MOVE_TIMEOUT		150
 #define E_CALENDAR_AUTO_MOVE_TIMEOUT_DELAY	2
 
+#define REPOSITION_TIMEOUT_KEY "e-calendar-reposition-timeout-id"
+
 static void e_calendar_dispose		(GObject	*object);
 static void e_calendar_realize		(GtkWidget	*widget);
 static void e_calendar_style_updated	(GtkWidget	*widget);
@@ -124,6 +126,50 @@ calitem_month_width_changed_cb (ECalendarItem *item,
 	gtk_widget_queue_resize (GTK_WIDGET (cal));
 }
 
+static GtkWidget *
+e_calendar_create_button (GtkArrowType arrow_type)
+{
+	GtkWidget *button, *pixmap;
+
+	button = gtk_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+	gtk_widget_show (button);
+
+	pixmap = gtk_arrow_new (arrow_type, GTK_SHADOW_NONE);
+	gtk_widget_show (pixmap);
+	gtk_container_add (GTK_CONTAINER (button), pixmap);
+
+	#if GTK_CHECK_VERSION (3, 20, 0)
+	{
+		GtkCssProvider *css_provider;
+		GtkStyleContext *style_context;
+		GError *error = NULL;
+
+		css_provider = gtk_css_provider_new ();
+		gtk_css_provider_load_from_data (css_provider,
+			"button.ecalendar {"
+			" min-height: 0px;"
+			" min-width: 0px;"
+			" padding: 0px;"
+			"}", -1, &error);
+		style_context = gtk_widget_get_style_context (button);
+		if (error == NULL) {
+			gtk_style_context_add_class (style_context, "ecalendar");
+			gtk_style_context_add_provider (
+				style_context,
+				GTK_STYLE_PROVIDER (css_provider),
+				GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+		} else {
+			g_warning ("%s: %s", G_STRFUNC, error->message);
+			g_clear_error (&error);
+		}
+		g_object_unref (css_provider);
+	}
+	#endif
+
+	return button;
+}
+
 static void
 e_calendar_class_init (ECalendarClass *class)
 {
@@ -151,7 +197,7 @@ e_calendar_init (ECalendar *cal)
 	GnomeCanvasGroup *canvas_group;
 	PangoFontDescription *small_font_desc;
 	PangoContext *pango_context;
-	GtkWidget *button, *pixmap;
+	GtkWidget *button;
 	AtkObject *a11y;
 
 	pango_context = gtk_widget_create_pango_context (GTK_WIDGET (cal));
@@ -181,9 +227,7 @@ e_calendar_init (ECalendar *cal)
 		G_CALLBACK (calitem_month_width_changed_cb), cal);
 
 	/* Create the arrow buttons to move to the previous/next month. */
-	button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_widget_show (button);
+	button = e_calendar_create_button (GTK_ARROW_LEFT);
 	g_signal_connect_swapped (
 		button, "pressed",
 		G_CALLBACK (e_calendar_on_prev_pressed), cal);
@@ -194,10 +238,6 @@ e_calendar_init (ECalendar *cal)
 		button, "clicked",
 		G_CALLBACK (e_calendar_on_prev_clicked), cal);
 
-	pixmap = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
-	gtk_widget_show (pixmap);
-	gtk_container_add (GTK_CONTAINER (button), pixmap);
-
 	cal->prev_item = gnome_canvas_item_new (
 		canvas_group,
 		gnome_canvas_widget_get_type (),
@@ -206,9 +246,7 @@ e_calendar_init (ECalendar *cal)
 	a11y = gtk_widget_get_accessible (button);
 	atk_object_set_name (a11y, _("Previous month"));
 
-	button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_widget_show (button);
+	button = e_calendar_create_button (GTK_ARROW_RIGHT);
 	g_signal_connect_swapped (
 		button, "pressed",
 		G_CALLBACK (e_calendar_on_next_pressed), cal);
@@ -219,10 +257,6 @@ e_calendar_init (ECalendar *cal)
 		button, "clicked",
 		G_CALLBACK (e_calendar_on_next_clicked), cal);
 
-	pixmap = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
-	gtk_widget_show (pixmap);
-	gtk_container_add (GTK_CONTAINER (button), pixmap);
-
 	cal->next_item = gnome_canvas_item_new (
 		canvas_group,
 		gnome_canvas_widget_get_type (),
@@ -232,9 +266,7 @@ e_calendar_init (ECalendar *cal)
 	atk_object_set_name (a11y, _("Next month"));
 
 	/* Create the arrow buttons to move to the previous/next year. */
-	button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_widget_show (button);
+	button = e_calendar_create_button (GTK_ARROW_LEFT);
 	g_signal_connect_swapped (
 		button, "pressed",
 		G_CALLBACK (e_calendar_on_prev_year_pressed), cal);
@@ -245,10 +277,6 @@ e_calendar_init (ECalendar *cal)
 		button, "clicked",
 		G_CALLBACK (e_calendar_on_prev_year_clicked), cal);
 
-	pixmap = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
-	gtk_widget_show (pixmap);
-	gtk_container_add (GTK_CONTAINER (button), pixmap);
-
 	cal->prev_item_year = gnome_canvas_item_new (
 		canvas_group,
 		gnome_canvas_widget_get_type (),
@@ -257,9 +285,7 @@ e_calendar_init (ECalendar *cal)
 	a11y = gtk_widget_get_accessible (button);
 	atk_object_set_name (a11y, _("Previous year"));
 
-	button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_widget_show (button);
+	button = e_calendar_create_button (GTK_ARROW_RIGHT);
 	g_signal_connect_swapped (
 		button, "pressed",
 		G_CALLBACK (e_calendar_on_next_year_pressed), cal);
@@ -270,10 +296,6 @@ e_calendar_init (ECalendar *cal)
 		button, "clicked",
 		G_CALLBACK (e_calendar_on_next_year_clicked), cal);
 
-	pixmap = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
-	gtk_widget_show (pixmap);
-	gtk_container_add (GTK_CONTAINER (button), pixmap);
-
 	cal->next_item_year = gnome_canvas_item_new (
 		canvas_group,
 		gnome_canvas_widget_get_type (),
@@ -309,6 +331,20 @@ e_calendar_new (void)
 	return cal;
 }
 
+static void
+cancel_pending_reposition_timeout (GObject *cal)
+{
+	guint reposition_timeout_id;
+
+	g_return_if_fail (E_IS_CALENDAR (cal));
+
+	reposition_timeout_id = GPOINTER_TO_UINT (g_object_get_data (cal, REPOSITION_TIMEOUT_KEY));
+	if (reposition_timeout_id) {
+		g_source_remove (reposition_timeout_id);
+		g_object_set_data (cal, REPOSITION_TIMEOUT_KEY, NULL);
+	}
+}
+
 static void
 e_calendar_dispose (GObject *object)
 {
@@ -324,6 +360,8 @@ e_calendar_dispose (GObject *object)
 		cal->timeout_id = 0;
 	}
 
+	cancel_pending_reposition_timeout (object);
+
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (e_calendar_parent_class)->dispose (object);
 }
@@ -373,13 +411,15 @@ e_calendar_get_preferred_width (GtkWidget *widget,
                                 gint *natural)
 {
 	ECalendar *cal;
+	GtkStyleContext *style_context;
 	GtkBorder padding;
 	gint col_width;
 
 	cal = E_CALENDAR (widget);
 
 	g_object_get ((cal->calitem), "column_width", &col_width, NULL);
-	gtk_style_context_get_padding (gtk_widget_get_style_context (widget), 0, &padding);
+	style_context = gtk_widget_get_style_context (widget);
+	gtk_style_context_get_padding (style_context, gtk_style_context_get_state (style_context), &padding);
 
 	*minimum = *natural = col_width * cal->min_cols + padding.left * 2;
 }
@@ -390,22 +430,25 @@ e_calendar_get_preferred_height (GtkWidget *widget,
                                  gint *natural)
 {
 	ECalendar *cal;
+	GtkStyleContext *style_context;
 	GtkBorder padding;
 	gint row_height;
 
 	cal = E_CALENDAR (widget);
 
 	g_object_get ((cal->calitem), "row_height", &row_height, NULL);
-	gtk_style_context_get_padding (gtk_widget_get_style_context (widget), 0, &padding);
+	style_context = gtk_widget_get_style_context (widget);
+	gtk_style_context_get_padding (style_context, gtk_style_context_get_state (style_context), &padding);
 
 	*minimum = *natural = row_height * cal->min_rows + padding.top * 2;
 }
 
-static void
-e_calendar_size_allocate (GtkWidget *widget,
-                          GtkAllocation *allocation)
+static gboolean
+e_calendar_reposition_timeout_cb (gpointer user_data)
 {
-	ECalendar *cal;
+	ECalendar *cal = user_data;
+	GtkWidget *widget;
+	GtkStyleContext *style_context;
 	GtkBorder padding;
 	GtkAllocation old_allocation;
 	PangoContext *pango_context;
@@ -414,13 +457,16 @@ e_calendar_size_allocate (GtkWidget *widget,
 	gdouble xthickness, ythickness, arrow_button_size, current_x, month_width;
 	gboolean is_rtl;
 
-	cal = E_CALENDAR (widget);
-	gtk_style_context_get_padding (gtk_widget_get_style_context (widget), 0, &padding);
+	g_return_val_if_fail (E_IS_CALENDAR (cal), FALSE);
+
+	g_object_set_data (G_OBJECT (cal), REPOSITION_TIMEOUT_KEY, NULL);
+
+	widget = GTK_WIDGET (cal);
+	style_context = gtk_widget_get_style_context (widget);
+	gtk_style_context_get_padding (style_context, gtk_style_context_get_state (style_context), &padding);
 	xthickness = padding.left;
 	ythickness = padding.top;
 
-	(*GTK_WIDGET_CLASS (e_calendar_parent_class)->size_allocate) (widget, allocation);
-
 	/* Set up Pango prerequisites */
 	pango_context = gtk_widget_get_pango_context (widget);
 	font_metrics = pango_context_get_metrics (
@@ -508,6 +554,23 @@ e_calendar_size_allocate (GtkWidget *widget,
 		NULL);
 
 	pango_font_metrics_unref (font_metrics);
+
+	return FALSE;
+}
+
+static void
+e_calendar_size_allocate (GtkWidget *widget,
+                          GtkAllocation *allocation)
+{
+	GObject *object;
+
+	(*GTK_WIDGET_CLASS (e_calendar_parent_class)->size_allocate) (widget, allocation);
+
+	object = G_OBJECT (widget);
+
+	cancel_pending_reposition_timeout (object);
+	g_object_set_data (object, REPOSITION_TIMEOUT_KEY, GUINT_TO_POINTER (
+		g_timeout_add (1, e_calendar_reposition_timeout_cb, widget)));
 }
 
 void
@@ -565,7 +628,7 @@ e_calendar_get_border_size (ECalendar *cal,
 	if (style_context) {
 		GtkBorder padding;
 
-		gtk_style_context_get_padding (style_context, 0, &padding);
+		gtk_style_context_get_padding (style_context, gtk_style_context_get_state (style_context), &padding);
 
 		*top    = padding.top;
 		*bottom = padding.top;
