@@ -36,6 +36,7 @@
 
 #include "e-http-request.h"
 #include "e-mail-display-popup-extension.h"
+#include "e-mail-notes.h"
 #include "e-mail-request.h"
 #include "em-composer-utils.h"
 #include "em-utils.h"
@@ -151,6 +152,10 @@ e_mail_display_claim_skipped_uri (EMailDisplay *mail_display,
 	g_return_if_fail (E_IS_MAIL_DISPLAY (mail_display));
 	g_return_if_fail (uri != NULL);
 
+	/* Do not store anything if the user doesn't want to see the notification */
+	if (!g_settings_get_boolean (mail_display->priv->settings, "notify-remote-content"))
+		return;
+
 	soup_uri = soup_uri_new (uri);
 	if (!soup_uri)
 		return;
@@ -261,7 +266,8 @@ mail_display_image_exists_in_cache (const gchar *image_uri)
 	gchar *hash;
 	gboolean exists = FALSE;
 
-	g_return_val_if_fail (emd_global_http_cache != NULL, FALSE);
+	if (!emd_global_http_cache)
+		return FALSE;
 
 	hash = g_compute_checksum_for_string (G_CHECKSUM_MD5, image_uri, -1);
 	filename = camel_data_cache_get_filename (
@@ -541,7 +547,7 @@ attachment_button_expanded (GObject *object,
 	EAttachmentButton *button = E_ATTACHMENT_BUTTON (object);
 	EMailDisplay *display = user_data;
 	WebKitDOMDocument *document;
-	WebKitDOMElement *element;
+	WebKitDOMElement *element, *iframe;
 	WebKitDOMCSSStyleDeclaration *css;
 	const gchar *attachment_part_id;
 	gchar *element_id;
@@ -587,6 +593,37 @@ attachment_button_expanded (GObject *object,
 		g_free (inner_html_data);
 	}
 
+	/* Hide/Show all the GtkWidgets inside an attachment, otherwise they could
+	 * be visible even if the wrapper is hidden. */
+	if ((iframe = webkit_dom_element_query_selector (element, "iframe", NULL))) {
+		WebKitDOMDocument *content_document;
+
+		content_document = webkit_dom_html_iframe_element_get_content_document
+			(WEBKIT_DOM_HTML_IFRAME_ELEMENT (iframe));
+
+		if (content_document) {
+			gint length, ii;
+			WebKitDOMNodeList *list;
+
+			list = webkit_dom_document_get_elements_by_tag_name (content_document, "object");
+			length = webkit_dom_node_list_get_length (list);
+
+			for (ii = 0; ii < length; ii++) {
+				WebKitDOMNode *item;
+
+				item = webkit_dom_node_list_item (list, ii);
+
+				css = webkit_dom_element_get_style (WEBKIT_DOM_ELEMENT (item));
+				if (expanded)
+					g_free (webkit_dom_css_style_declaration_remove_property (css, "display", NULL));
+				else
+					webkit_dom_css_style_declaration_set_property (css, "display", "none", "", NULL);
+				g_clear_object (&css);
+			}
+			g_object_unref (list);
+		}
+	}
+
 	/* Show or hide the DIV which contains
 	 * the attachment (iframe, image...). */
 	css = webkit_dom_element_get_style (element);
@@ -776,6 +813,7 @@ mail_display_plugin_widget_requested (WebKitWebView *web_view,
 	 * that URI identifies the attachment itself */
 	if (E_IS_ATTACHMENT_BUTTON (widget)) {
 		EMailPartAttachment *empa = (EMailPartAttachment *) part;
+		EAttachment *attachment;
 		gchar *attachment_part_id;
 
 		if (empa->attachment_view_part_id)
@@ -789,6 +827,28 @@ mail_display_plugin_widget_requested (WebKitWebView *web_view,
 			G_OBJECT (widget), "attachment_id",
 			g_strdup (attachment_part_id),
 			(GDestroyNotify) g_free);
+
+		attachment = e_mail_part_attachment_ref_attachment (empa);
+		if (attachment && e_attachment_is_mail_note (attachment)) {
+			CamelFolder *folder;
+			const gchar *message_uid;
+
+			folder = e_mail_part_list_get_folder (display->priv->part_list);
+			message_uid = e_mail_part_list_get_message_uid (display->priv->part_list);
+
+			if (folder && message_uid) {
+				CamelMessageInfo *info;
+
+				info = camel_folder_get_message_info (folder, message_uid);
+				if (info) {
+					if (!camel_message_info_user_flag (info, E_MAIL_NOTES_USER_FLAG))
+						camel_message_info_set_user_flag (info, E_MAIL_NOTES_USER_FLAG, TRUE);
+					camel_message_info_unref (info);
+				}
+			}
+		}
+
+		g_clear_object (&attachment);
 	} else {
 		object_uri = g_strdup (part_id);
 	}
@@ -1980,14 +2040,23 @@ e_mail_display_init (EMailDisplay *display)
 		E_WEB_VIEW (display), E_TYPE_STOCK_REQUEST);
 
 	if (emd_global_http_cache == NULL) {
-		user_cache_dir = e_get_user_cache_dir ();
-		emd_global_http_cache = camel_data_cache_new (user_cache_dir, NULL);
+		GError *error = NULL;
 
-		/* cache expiry - 2 hour access, 1 day max */
-		camel_data_cache_set_expire_age (
-			emd_global_http_cache, 24 * 60 * 60);
-		camel_data_cache_set_expire_access (
-			emd_global_http_cache, 2 * 60 * 60);
+		user_cache_dir = e_get_user_cache_dir ();
+		emd_global_http_cache = camel_data_cache_new (user_cache_dir, &error);
+
+		if (emd_global_http_cache) {
+			/* cache expiry - 2 hour access, 1 day max */
+			camel_data_cache_set_expire_age (
+				emd_global_http_cache, 24 * 60 * 60);
+			camel_data_cache_set_expire_access (
+				emd_global_http_cache, 2 * 60 * 60);
+		} else {
+			e_alert_submit (
+				E_ALERT_SINK (display), "mail:folder-open",
+				error ? error->message : _("Unknown error"), NULL);
+			g_clear_error (&error);
+		}
 	}
 
 	g_mutex_init (&display->priv->remote_content_lock);
