@@ -1999,6 +1999,14 @@ static GtkToggleActionEntry mail_toggle_entries[] = {
 	  NULL,  /* Handled by property bindings */
 	  FALSE },
 
+	{ "mail-show-junk",
+	  NULL,
+	  N_("Show _Junk Messages"),
+	  NULL,
+	  N_("Show junk messages with a red line through them"),
+	  NULL,  /* Handled by property bindings */
+	  FALSE },
+
 	{ "mail-threads-group-by",
 	  NULL,
 	  N_("_Group By Threads"),
@@ -2079,6 +2087,13 @@ static GtkRadioActionEntry mail_filter_entries[] = {
 	  NULL,  /* XXX Add a tooltip! */
 	  MAIL_FILTER_MESSAGES_WITH_ATTACHMENTS },
 
+	{ "mail-filter-messages-with-notes",
+	  "evolution-memos",
+	  N_("Messages with Notes"),
+	  NULL,
+	  NULL,  /* XXX Add a tooltip! */
+	  MAIL_FILTER_MESSAGES_WITH_NOTES },
+
 	{ "mail-filter-no-label",
 	  NULL,
 	  N_("No Label"),
@@ -2098,7 +2113,15 @@ static GtkRadioActionEntry mail_filter_entries[] = {
 	  N_("Unread Messages"),
 	  NULL,
 	  NULL,  /* XXX Add a tooltip! */
-	  MAIL_FILTER_UNREAD_MESSAGES }
+	  MAIL_FILTER_UNREAD_MESSAGES },
+
+	{ "mail-filter-message-thread",
+	  NULL,
+	  N_("Message Thread"),
+	  NULL,
+	  NULL,  /* XXX Add a tooltip! */
+	  MAIL_FILTER_MESSAGE_THREAD }
+
 };
 
 static GtkRadioActionEntry mail_search_entries[] = {
@@ -2262,6 +2285,11 @@ e_mail_shell_view_actions_init (EMailShellView *mail_shell_view)
 		ACTION (MAIL_SHOW_DELETED), "active",
 		G_SETTINGS_BIND_DEFAULT);
 
+	g_settings_bind (
+		settings, "show-junk",
+		ACTION (MAIL_SHOW_JUNK), "active",
+		G_SETTINGS_BIND_DEFAULT);
+
 	g_settings_bind (
 		settings, "layout",
 		ACTION (MAIL_VIEW_VERTICAL), "current-value",
@@ -2324,6 +2352,12 @@ e_mail_shell_view_actions_init (EMailShellView *mail_shell_view)
 		G_BINDING_BIDIRECTIONAL |
 		G_BINDING_SYNC_CREATE);
 
+	e_binding_bind_property (
+		ACTION (MAIL_SHOW_JUNK), "active",
+		mail_view, "show-junk",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
+
 	e_binding_bind_property (
 		shell_backend, "busy",
 		ACTION (MAIL_STOP), "sensitive",
@@ -2600,7 +2634,7 @@ e_mail_shell_view_update_search_filter (EMailShellView *mail_shell_view)
 	/* Use any action in the group; doesn't matter which. */
 	e_action_combo_box_set_action (combo_box, radio_action);
 
-	ii = MAIL_FILTER_UNREAD_MESSAGES;
+	ii = MAIL_FILTER_MESSAGES_NOT_JUNK;
 	e_action_combo_box_add_separator_after (combo_box, ii);
 
 	ii = MAIL_FILTER_READ_MESSAGES;
